@@ -1,4 +1,4 @@
-from __future__ import division
+from __future__ import division, absolute_import, print_function
 
 __copyright__ = "Copyright (C) 2009-2013 Andreas Kloeckner"
 
@@ -27,12 +27,17 @@ import operator
 import sys
 
 from pytools.decorator import decorator
+import six
+from six.moves import range, zip, intern, input
+from functools import reduce
 
 try:
     decorator_module = __import__("decorator", level=0)
 except TypeError:
     # this must be Python 2.4
     my_decorator = decorator
+except ImportError:
+    my_decorator = decorator
 else:
     my_decorator = decorator_module.decorator
 
@@ -61,7 +66,7 @@ def levi_civita(tup):
 def factorial(n):
     from operator import mul
     assert n == int(n)
-    return reduce(mul, (i for i in xrange(1, n+1)), 1)
+    return reduce(mul, (i for i in range(1, n+1)), 1)
 
 
 def perm(n, k):
@@ -133,8 +138,8 @@ class RecordWithoutPickling(object):
         if valuedict is not None:
             kwargs.update(valuedict)
 
-        for key, value in kwargs.iteritems():
-            if not key in exclude:
+        for key, value in six.iteritems(kwargs):
+            if key not in exclude:
                 fields.add(key)
                 setattr(self, key, value)
 
@@ -181,7 +186,7 @@ class Record(RecordWithoutPickling):
         except AttributeError:
             self.__class__.fields = fields = set()
 
-        for key, value in valuedict.iteritems():
+        for key, value in six.iteritems(valuedict):
             fields.add(key)
             setattr(self, key, value)
 
@@ -230,13 +235,13 @@ class DictionaryWithDefault(object):
         return True
 
     def iterkeys(self):
-        return self._Dictionary.iterkeys()
+        return six.iterkeys(self._Dictionary)
 
     def __iter__(self):
         return self._Dictionary.__iter__()
 
     def iteritems(self):
-        return self._Dictionary.iteritems()
+        return six.iteritems(self._Dictionary)
 
 # }}}
 
@@ -283,17 +288,17 @@ class DependentDictionary(object):
     def __setitem__(self, key, value):
         self._Dictionary[key] = value
 
-    def genuineKeys(self):
-        return self._Dictionary.keys()
+    def genuineKeys(self):  # noqa
+        return list(self._Dictionary.keys())
 
     def iteritems(self):
-        return self._Dictionary.iteritems()
+        return six.iteritems(self._Dictionary)
 
     def iterkeys(self):
-        return self._Dictionary.iterkeys()
+        return six.iterkeys(self._Dictionary)
 
     def itervalues(self):
-        return self._Dictionary.itervalues()
+        return six.itervalues(self._Dictionary)
 
 # }}}
 
@@ -308,13 +313,13 @@ def one(iterable):
     """
     it = iter(iterable)
     try:
-        v = it.next()
+        v = next(it)
     except StopIteration:
         raise ValueError("empty iterable passed to 'one()'")
 
     def no_more():
         try:
-            it.next()
+            next(it)
             raise ValueError("iterable with more than one entry passed to 'one()'")
         except StopIteration:
             return True
@@ -327,7 +332,7 @@ def one(iterable):
 def is_single_valued(iterable, equality_pred=operator.eq):
     it = iter(iterable)
     try:
-        first_item = it.next()
+        first_item = next(it)
     except StopIteration:
         raise ValueError("empty iterable passed to 'single_valued()'")
 
@@ -351,7 +356,7 @@ def single_valued(iterable, equality_pred=operator.eq):
     """
     it = iter(iterable)
     try:
-        first_item = it.next()
+        first_item = next(it)
     except StopIteration:
         raise ValueError("empty iterable passed to 'single_valued()'")
 
@@ -385,7 +390,7 @@ def memoize(*args, **kwargs):
 
     if use_kw:
         def default_key_func(*inner_args, **inner_kwargs):
-            return inner_args, frozenset(inner_kwargs.iteritems())
+            return inner_args, frozenset(six.iteritems(inner_kwargs))
     else:
         default_key_func = None
 
@@ -394,7 +399,7 @@ def memoize(*args, **kwargs):
     if kwargs:
         raise TypeError(
             "memoize received unexpected keyword arguments: %s"
-            % ", ".join(kwargs.keys()))
+            % ", ".join(list(kwargs.keys())))
 
     if key_func is not None:
         @my_decorator
@@ -454,7 +459,7 @@ def memoize_method(method):
 
     def wrapper(self, *args, **kwargs):
         if kwargs:
-            key = (_HasKwargs, frozenset(kwargs.iteritems())) + args
+            key = (_HasKwargs, frozenset(six.iteritems(kwargs))) + args
         else:
             key = args
 
@@ -495,7 +500,7 @@ def memoize_on_first_arg(function):
 
     def wrapper(obj, *args, **kwargs):
         if kwargs:
-            key = (_HasKwargs, frozenset(kwargs.iteritems())) + args
+            key = (_HasKwargs, frozenset(six.iteritems(kwargs))) + args
         else:
             key = args
 
@@ -549,7 +554,9 @@ def memoize_method_with_uncached(uncached_args=[], uncached_kwargs=set()):
                 for name in uncached_kwargs:
                     cache_kwargs.pop(name, None)
 
-                key = (_HasKwargs, frozenset(cache_kwargs.iteritems())) + cache_args
+                key = (
+                        (_HasKwargs, frozenset(six.iteritems(cache_kwargs)))
+                        + cache_args)
             else:
                 key = cache_args
 
@@ -585,13 +592,17 @@ def memoize_method_nested(inner):
     Requires Python 2.5 or newer.
     """
 
+    from warnings import warn
+    warn("memoize_method_nested is deprecated. Use @memoize_in(self, 'identifier') "
+            "instead", DeprecationWarning, stacklevel=2)
+
     from functools import wraps
     cache_dict_name = intern("_memoize_inner_dic_%s_%s_%d"
-            % (inner.__name__, inner.func_code.co_filename,
-                inner.func_code.co_firstlineno))
+            % (inner.__name__, inner.__code__.co_filename,
+                inner.__code__.co_firstlineno))
 
-    from inspect import currentframe, getouterframes
-    outer_frame = getouterframes(currentframe())[1][0]
+    from inspect import currentframe
+    outer_frame = currentframe().f_back
     cache_context = outer_frame.f_locals.get("memoize_cache_context")
     if cache_context is None:
         cache_context = outer_frame.f_locals.get("self")
@@ -613,6 +624,37 @@ def memoize_method_nested(inner):
 
     return new_inner
 
+
+class memoize_in(object):  # noqa
+    """Adds a cache to a function nested inside a method. The cache is attached
+    to *memoize_cache_context* (if it exists) or *self* in the outer (method)
+    namespace.
+
+    Requires Python 2.5 or newer.
+    """
+
+    def __init__(self, container, identifier):
+        key = "_pytools_memoize_in_dict_for_"+identifier
+        try:
+            self.cache_dict = getattr(container, key)
+        except AttributeError:
+            self.cache_dict = {}
+            setattr(container, key, self.cache_dict)
+
+    def __call__(self, inner):
+        from functools import wraps
+
+        @wraps(inner)
+        def new_inner(*args):
+            try:
+                return self.cache_dict[args]
+            except KeyError:
+                result = inner(*args)
+                self.cache_dict[args] = result
+                return result
+
+        return new_inner
+
 # }}}
 
 
@@ -650,7 +692,7 @@ def monkeypatch_class(name, bases, namespace):
 
     assert len(bases) == 1, "Exactly one base class required"
     base = bases[0]
-    for name, value in namespace.iteritems():
+    for name, value in six.iteritems(namespace):
         if name != "__metaclass__":
             setattr(base, name, value)
     return base
@@ -676,7 +718,7 @@ def shift(vec, dist):
 
     result = vec[:]
 
-    N = len(vec)
+    N = len(vec)  # noqa
     dist = dist % N
 
     # modulo only returns positive distances!
@@ -716,7 +758,7 @@ def linear_combination(coefficients, vectors):
 def common_prefix(iterable, empty=None):
     it = iter(iterable)
     try:
-        pfx = it.next()
+        pfx = next(it)
     except StopIteration:
         return empty
 
@@ -732,7 +774,7 @@ def common_prefix(iterable, empty=None):
 
 
 def decorate(function, list):
-    return map(lambda x: (x, function(x)), list)
+    return [(x, function(x)) for x in list]
 
 
 def partition(criterion, list):
@@ -781,7 +823,7 @@ except AttributeError:
 
 def reverse_dictionary(the_dict):
     result = {}
-    for key, value in the_dict.iteritems():
+    for key, value in six.iteritems(the_dict):
         if value in result:
             raise RuntimeError(
                     "non-reversible mapping, duplicate key '%s'" % value)
@@ -884,7 +926,7 @@ def find_max_where(predicate, prec=1e-5, initial_guess=1, fail_bound=1e38):
 def argmin2(iterable, return_value=False):
     it = iter(iterable)
     try:
-        current_argmin, current_min = it.next()
+        current_argmin, current_min = next(it)
     except StopIteration:
         raise ValueError("argmin of empty iterable")
 
@@ -902,7 +944,7 @@ def argmin2(iterable, return_value=False):
 def argmax2(iterable, return_value=False):
     it = iter(iterable)
     try:
-        current_argmax, current_max = it.next()
+        current_argmax, current_max = next(it)
     except StopIteration:
         raise ValueError("argmax of empty iterable")
 
@@ -963,7 +1005,7 @@ def average(iterable):
     it = iterable.__iter__()
 
     try:
-        sum = it.next()
+        sum = next(it)
         count = 1
     except StopIteration:
         raise ValueError("empty average")
@@ -1033,11 +1075,11 @@ def indices_in_shape(shape):
     if len(shape) == 0:
         yield ()
     elif len(shape) == 1:
-        for i in xrange(0, shape[0]):
+        for i in range(0, shape[0]):
             yield (i,)
     else:
         remainder = shape[1:]
-        for i in xrange(0, shape[0]):
+        for i in range(0, shape[0]):
             for rest in indices_in_shape(remainder):
                 yield (i,)+rest
 
@@ -1193,7 +1235,7 @@ def get_read_from_map_from_permutation(original, permuted):
     """
     assert len(original) == len(permuted)
     where_in_original = dict(
-            (original[i], i) for i in xrange(len(original)))
+            (original[i], i) for i in range(len(original)))
     assert len(where_in_original) == len(original)
     return tuple(where_in_original[pi] for pi in permuted)
 
@@ -1217,7 +1259,7 @@ def get_write_to_map_from_permutation(original, permuted):
     assert len(original) == len(permuted)
 
     where_in_permuted = dict(
-            (permuted[i], i) for i in xrange(len(permuted)))
+            (permuted[i], i) for i in range(len(permuted)))
 
     assert len(where_in_permuted) == len(permuted)
     return tuple(where_in_permuted[oi] for oi in original)
@@ -1351,7 +1393,7 @@ def string_histogram(iterable, min_value=None, max_value=None,
             try:
                 bins[bin_nr] += 1
             except:
-                print value, bin_nr, bin_starts
+                print(value, bin_nr, bin_starts)
                 raise
 
     from math import floor, ceil
@@ -1420,26 +1462,26 @@ class CPyUserInterface(object):
         self.doc = doc
 
     def show_usage(self, progname):
-        print "usage: %s <FILE-OR-STATEMENTS>" % progname
-        print
-        print "FILE-OR-STATEMENTS may either be Python statements of the form"
-        print "'variable1 = value1; variable2 = value2' or the name of a file"
-        print "containing such statements. Any valid Python code may be used"
-        print "on the command line or in a command file. If new variables are"
-        print "used, they must start with 'user_' or just '_'."
-        print
-        print "The following variables are recognized:"
+        print("usage: %s <FILE-OR-STATEMENTS>" % progname)
+        print()
+        print("FILE-OR-STATEMENTS may either be Python statements of the form")
+        print("'variable1 = value1; variable2 = value2' or the name of a file")
+        print("containing such statements. Any valid Python code may be used")
+        print("on the command line or in a command file. If new variables are")
+        print("used, they must start with 'user_' or just '_'.")
+        print()
+        print("The following variables are recognized:")
         for v in sorted(self.variables):
-            print "  %s = %s" % (v, self.variables[v])
+            print("  %s = %s" % (v, self.variables[v]))
             if v in self.doc:
-                print "    %s" % self.doc[v]
+                print("    %s" % self.doc[v])
 
-        print
-        print "The following constants are supplied:"
+        print()
+        print("The following constants are supplied:")
         for c in sorted(self.constants):
-            print "  %s = %s" % (c, self.constants[c])
+            print("  %s = %s" % (c, self.constants[c]))
             if c in self.doc:
-                print "    %s" % self.doc[c]
+                print("    %s" % self.doc[c])
 
     def gather(self, argv=None):
         import sys
@@ -1530,7 +1572,7 @@ def typedump(val, max_seq=5, special_handlers={}):
             return "{%s}" % (
                     ", ".join(
                         "%r: %s" % (str(k), typedump(v))
-                        for k, v in val.iteritems()))
+                        for k, v in six.iteritems(val)))
 
         try:
             if len(val) > max_seq:
@@ -1564,9 +1606,9 @@ def invoke_editor(s, filename="edit.txt", descr="the file"):
         p = Popen([os.environ["EDITOR"], full_name])
         os.waitpid(p.pid, 0)[1]
     else:
-        print "(Set the EDITOR environment variable to be " \
-                "dropped directly into an editor next time.)"
-        raw_input("Edit %s at %s now, then hit [Enter]:"
+        print("(Set the EDITOR environment variable to be "
+                "dropped directly into an editor next time.)")
+        input("Edit %s at %s now, then hit [Enter]:"
                 % (descr, full_name))
 
     inf = open(full_name, "r")
@@ -1615,7 +1657,7 @@ class ProgressBar:
                 self.speed_meas_start_done = self.done
 
             if self.time_per_step is not None:
-                eta_str = "%6.1fs" % max(
+                eta_str = "%7.1fs " % max(
                         0, (self.total-self.done) * self.time_per_step)
             else:
                 eta_str = "?"
@@ -1641,6 +1683,12 @@ class ProgressBar:
         self.set_progress(self.total)
         sys.stderr.write("\n")
 
+    def __enter__(self):
+        self.draw()
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.finished()
+
 # }}}
 
 
@@ -1754,6 +1802,8 @@ class UniqueNameGenerator:
             if not self.is_name_conflicting(var_name):
                 break
 
+        var_name = intern(var_name)
+
         self.existing_names.add(var_name)
         return var_name
 
