@@ -1,7 +1,10 @@
 from __future__ import division, with_statement
+from __future__ import absolute_import
 
 import pytest  # noqa
 import sys  # noqa
+from six.moves import range
+from six.moves import zip
 
 
 def test_persistent_dict():
@@ -19,19 +22,19 @@ def test_persistent_dict():
     keys = [(randrange(2000), rand_str(), None) for i in range(20)]
     values = [randrange(2000) for i in range(20)]
 
-    d = dict(zip(keys, values))
+    d = dict(list(zip(keys, values)))
 
     for k, v in zip(keys, values):
         pdict[k] = v
         pdict.store(k, v, info_files={"hey": str(v)})
 
-    for k, v in d.items():
+    for k, v in list(d.items()):
         assert d[k] == pdict[k]
 
     for k, v in zip(keys, values):
         pdict.store(k, v+1, info_files={"hey": str(v)})
 
-    for k, v in d.items():
+    for k, v in list(d.items()):
         assert d[k] + 1 == pdict[k]
 
 
